import hmac
from zope.component import getUtility
from zope.interface import implements
from AccessControl import getSecurityManager
from zExceptions import Forbidden
from ZPublisher.HTTPRequest import HTTPRequest
from Products.Five import BrowserView
from plone.keyring.interfaces import IKeyManager
from plone.protect.interfaces import IAuthenticatorView

try:
    from hashlib import sha1 as sha
except ImportError:
    import sha


def _getUserName():
    user = getSecurityManager().getUser()
    if user is None:
        return "Anonymous User"
    return user.getUserName()


def _is_equal(val1, val2):
    """
    constant time comparison
    """
    if not isinstance(val1, basestring) or not isinstance(val2, basestring):
        return False
    if len(val1) != len(val2):
        return False
    result = 0
    for x, y in zip(val1, val2):
        result |= ord(x) ^ ord(y)
    return result == 0


def _verify(request, extra='', name='_authenticator'):
    auth = request.get(name)
    if auth is None:
        return False

    manager = getUtility(IKeyManager)
    ring = manager[u"_system"]
    user = _getUserName()

    for key in ring:
        if key is None:
            continue
        correct = hmac.new(key, user + extra, sha).hexdigest()
        if _is_equal(correct, auth):
            return True

    return False


def createToken(extra=''):
    manager = getUtility(IKeyManager)
    secret = manager.secret()
    user = _getUserName()
    return hmac.new(secret, user + extra, sha).hexdigest()


class AuthenticatorView(BrowserView):
    implements(IAuthenticatorView)

    def token(self, extra=''):
        return createToken(extra)

    def authenticator(self, extra='', name='_authenticator'):
        auth = createToken(extra)
        return '<input type="hidden" name="%s" value="%s"/>' % (
                name, auth)

    def verify(self, extra='', name="_authenticator"):
        return _verify(self.request, extra=extra, name=name)


def check(request, extra='', name="_authenticator"):
    if isinstance(request, HTTPRequest):
        if not _verify(request, extra=extra, name=name):
            raise Forbidden('Form authenticator is invalid.')


def CustomCheckAuthenticator(extra='', name='_authenticator'):
    def _check(request):
        return check(request, extra=extra, name=name)
    return _check


__all__ = ["AuthenticatorView", "check", "createToken",
           "CustomCheckAuthenticator"]



from unittest import TestCase
from zope.component import getGlobalSiteManager
from zope.interface import implements
from plone.keyring.interfaces import IKeyManager
from AccessControl.SecurityManagement import newSecurityManager
from AccessControl.SecurityManagement import noSecurityManager
from AccessControl.User import User


class MockRequest(dict):

    def __init__(self, URL=None, *args, **kwargs):
        super(MockRequest, self).__init__(*args, **kwargs)
        self.environ = {}
        self.URL = URL

    def setReferer(self, url):
        self.environ['HTTP_REFERER'] = url


class MockKeyManager:
    implements(IKeyManager)

    keys = ["one", "two", "three", "four", "five"]

    def secret(self):
        return self.keys[0]

    def __getitem__(self, key):
        return self.keys


class KeyringTestCase(TestCase):

    def setUp(self):
        self.sm = getGlobalSiteManager()
        self.manager = MockKeyManager()
        self.sm.registerUtility(self.manager,
            provided=IKeyManager, event=False)
        # Tests modify the user object so we better make sure it is *our*
        # user object and not the built-in Anonymous User.
        newSecurityManager(None, User('dummy', 'secret', (), ()))

    def tearDown(self):
        self.sm.unregisterUtility(self.manager, provided=IKeyManager)
        noSecurityManager()



from zope.interface import Interface


class IAuthenticatorView(Interface):

    def token():
        """return token value"""

    def authenticator():
        """Return an xhtml snippet which sets an authenticator.

        This must be included inside a <form> element.
        """

    def verify():
        """
        Verify if the request contains a valid authenticator.
        """


class IDisableCSRFProtection(Interface):
    """Be able to disable on a per-request basis.

    In this version of plone.protect it does nothing.

    Forward port from plone.protect 3, so you do not need code like this
    when you want to disable csrf protection:

        try:
            from plone.protect.interfaces import IDisableCSRFProtection
        except ImportError:
            IDisableCSRFProtection = None
        if IDisableCSRFProtection is not None:
            alsoProvides(request, IDisableCSRFProtection)
    """



from setuptools import setup, find_packages

version = '2.0.3'

setup(name='plone.protect',
      version=version,
      description="Security for browser forms",
      long_description=open("README.rst").read() + "\n" + \
                       open("CHANGES.txt").read(),
      classifiers=[
          "Environment :: Web Environment",
          "Framework :: Plone",
          "Framework :: Zope2",
          "License :: OSI Approved :: BSD License",
          "Operating System :: OS Independent",
          "Programming Language :: Python",
        ],
      keywords='zope security CSRF',
      author='Plone Foundation',
      author_email='plone-developers@lists.sourceforge.net',
      url='http://pypi.python.org/pypi/plone.protect',
      license='BSD',
      packages=find_packages(exclude=['ez_setup']),
      namespace_packages=['plone', ],
      include_package_data=True,
      zip_safe=False,
      install_requires=[
          'setuptools',
          'plone.keyring > 1.0',
          'zope.component',
          'zope.interface',
          'Zope2',
      ],
      )
